﻿using CreateLearnerExample.Models;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace CreateLearnerExample.Services.V1
{
    public class LearnerService
    {
        private HttpClient _client;

        public LearnerService(HttpClient client)
        {
            _client = client;
        }

        /// <summary>
        /// Get a single learner object associated with a specific ID within the centre
        /// </summary>
        /// <param name="sessionId">Guid generated upon authentication</param>
        /// <param name="learnerId">ID of the user within OneFile</param>
        /// <returns>A learner object</returns>
        public async Task<User> GetLearner(Guid sessionId, int learnerId)
        {
            if (sessionId == Guid.Empty)
            {
                throw new ArgumentNullException(nameof(sessionId) + " Cannot be null");
            }

            _client.DefaultRequestHeaders.Clear();
            _client.DefaultRequestHeaders.Add("X-SessionID", sessionId.ToString());

            HttpResponseMessage response =
                await _client.GetAsync($"{Program.HOST}/api/v1/Learner/GetLearner/{learnerId}");

            if (response.IsSuccessStatusCode)
            {
                var responseMessage = await response.Content.ReadAsStringAsync();

                return JsonConvert.DeserializeObject<User>(responseMessage);
            }
            throw new UnauthorizedAccessException("Unable to retrieve learners");
        }

        /// <summary>
        /// Get a list of all learners within the organisation
        /// </summary>
        /// <param name="sessionId">Guid generated upon authentication</param>
        /// <param name="organisationId">ID of an organisation (Centre)</param>
        /// <returns>List of learner objects</returns>
        public async Task<List<User>> GetLearners(Guid sessionId, int organisationId)
        {
            if (sessionId == Guid.Empty)
            {
                throw new ArgumentNullException(nameof(sessionId) + " Cannot be null");
            }

            _client.DefaultRequestHeaders.Clear();
            // X-SessionID header is required on each request
            _client.DefaultRequestHeaders.Add("X-SessionID", sessionId.ToString());

            HttpResponseMessage response =
                await _client.GetAsync($"{Program.HOST}/api/v1/Learner/GetAllLearners/{organisationId}");

            if (response.IsSuccessStatusCode)
            {
                string responseMessage = await response.Content.ReadAsStringAsync();

                return JsonConvert.DeserializeObject<List<User>>(responseMessage);
            }

            throw new UnauthorizedAccessException("Unable to retrieve learners");
        }

        /// <summary>
        /// Create a learner with the minimum data
        /// </summary>
        /// <param name="sessionId">Guid generated upon authentication</param>
        /// <param name="learner">An object that contains the learners details</param>
        /// <returns>new Learner ID</returns>
        public async Task<int> CreateLearner(Guid sessionId, User learner)
        {
            if (sessionId == Guid.Empty)
            {
                throw new ArgumentNullException($"{nameof(sessionId)} Cannot be null");
            }

            _client.DefaultRequestHeaders.Clear();
            _client.DefaultRequestHeaders.Add("X-SessionID", sessionId.ToString());
            
            var learnerJsonValue = JsonConvert.SerializeObject(learner);
            HttpResponseMessage response =
                _client.PostAsync($"{Program.HOST}/api/v1/Learner/CreateLearner/{learner.OrganisationID}", new StringContent(learnerJsonValue, Encoding.UTF8, "application/json")).Result;
            
            string responseMessage = await response.Content.ReadAsStringAsync();

            if (response.IsSuccessStatusCode)
            {

                int.TryParse(await response.Content.ReadAsStringAsync(), out int newUserId);
                return newUserId;
            }
            throw new UnauthorizedAccessException($"Unable to create a learner - StatusCode ({response.StatusCode})");
        }
    }
}
